# MODULES
from typing import Dict as _Dict, List as _List, Optional as _Optional

# PYDANTIC
from pydantic import BaseModel as _BaseModel, ConfigDict as _ConfigDict, Field as _Field


class ContactSchema(_BaseModel):
    name: _Optional[str] = _Field(default=None)
    email: _Optional[str] = _Field(default=None)


class OpenApiSchema(_BaseModel):
    model_config = _ConfigDict(from_attributes=True)

    description: _Optional[str] = _Field(default=None)
    contact: _Optional[ContactSchema] = _Field(default=None)
    swagger_favicon_url: _Optional[str] = _Field(default=None)
    redoc_favicon_url: _Optional[str] = _Field(default=None)
    tags: _List[_Dict[str, str]] = _Field(default_factory=lambda: [])



from alphaz_next.core.responses.file_response import FileResponse as FileResponse
from alphaz_next.core.responses.html_response import HTMLResponse as HTMLResponse
from alphaz_next.core.responses.json_response import JSONResponse as JSONResponse
from alphaz_next.core.responses.orjson_response import ORJSONResponse as ORJSONResponse
from alphaz_next.core.responses.plaintext_response import (
    PlainTextResponse as PlainTextResponse,
)
from alphaz_next.core.responses.redirect_response import (
    RedirectResponse as RedirectResponse,
)
from alphaz_next.core.responses.streaming_response import (
    StreamingResponse as StreamingResponse,
)
from alphaz_next.core.responses.ujson_response import UJSONResponse as UJSONResponse
from alphaz_next.core.responses.response import Response as Response



# MODULES
from typing import List as _List

# PYDANTIC
from pydantic import BaseModel as _BaseModel, computed_field as _computed_field


class UserBaseSchema(_BaseModel):
    """
    Represents a base schema for a user.
    """

    @_computed_field  # type: ignore
    @property
    def permissions(self) -> _List[str]:
        raise NotImplementedError(
            "This method must be implemented in the derived class."
        )



# MODULES
from typing import (
    List as _List,
    Optional as _Optional,
    Type as _Type,
)

# PYSQL_REPO
from pysql_repo import DataBase as _DataBase
from pysql_repo._database_base import (
    DataBaseConfigTypedDict as _DataBaseConfigTypedDict,
)

# SQLALCHEMY
from sqlalchemy import MetaData as _MetaData
from sqlalchemy.orm import DeclarativeBase as _DeclarativeBase

# OPENTELEMETRY
from opentelemetry.instrumentation.sqlalchemy import (
    SQLAlchemyInstrumentor as _SQLAlchemyInstrumentor,
)


class DataBase(_DataBase):

    def __init__(
        self,
        databases_config: _DataBaseConfigTypedDict,
        base: _Type[_DeclarativeBase],
        metadata_views: _Optional[_List[_MetaData]] = None,
        autoflush: bool = False,
        expire_on_commit: bool = False,
        echo: bool = False,
    ) -> None:
        super().__init__(
            databases_config=databases_config,
            base=base,
            metadata_views=metadata_views,
            autoflush=autoflush,
            expire_on_commit=expire_on_commit,
            echo=echo,
        )

        _SQLAlchemyInstrumentor().instrument(engine=self._engine)
